@@ -1,20 +1,18 @@
 import socket
-import sys
 from redis.connection import (Connection, SYM_STAR, SYM_DOLLAR, SYM_EMPTY,
-                              SYM_CRLF, b)
+                              SYM_CRLF)
 from redis._compat import imap
 from base import Benchmark
 
 
 class StringJoiningConnection(Connection):
-    def send_packed_command(self, command):
+    def send_packed_command(self, command, check_health=True):
         "Send an already packed command to the Redis server"
         if not self._sock:
             self.connect()
         try:
             self._sock.sendall(command)
-        except socket.error:
-            e = sys.exc_info()[1]
+        except socket.error as e:
             self.disconnect()
             if len(e.args) == 1:
                 _errno, errmsg = 'UNKNOWN', e.args[0]
@@ -22,9 +20,9 @@ class StringJoiningConnection(Connection):
                 _errno, errmsg = e.args
             raise ConnectionError("Error %s while writing to socket. %s." %
                                   (_errno, errmsg))
-        except Exception as e:
+        except Exception:
             self.disconnect()
-            raise e
+            raise
 
     def pack_command(self, *args):
         "Pack a series of arguments into a value Redis command"
@@ -38,7 +36,7 @@ class StringJoiningConnection(Connection):
 
 
 class ListJoiningConnection(Connection):
-    def send_packed_command(self, command):
+    def send_packed_command(self, command, check_health=True):
         if not self._sock:
             self.connect()
         try:
@@ -46,8 +44,7 @@ class ListJoiningConnection(Connection):
                 command = [command]
             for item in command:
                 self._sock.sendall(item)
-        except socket.error:
-            e = sys.exc_info()[1]
+        except socket.error as e:
             self.disconnect()
             if len(e.args) == 1:
                 _errno, errmsg = 'UNKNOWN', e.args[0]
@@ -55,9 +52,9 @@ class ListJoiningConnection(Connection):
                 _errno, errmsg = e.args
             raise ConnectionError("Error %s while writing to socket. %s." %
                                   (_errno, errmsg))
-        except Exception as e:
+        except Exception:
             self.disconnect()
-            raise e
+            raise
 
     def pack_command(self, *args):
         output = []
