@@ -26,6 +26,13 @@ class TestLock(object):
 
     def test_lock_token(self, r):
         lock = self.get_lock(r, 'foo')
+        self._test_lock_token(r, lock)
+
+    def test_lock_token_thread_local_false(self, r):
+        lock = self.get_lock(r, 'foo', thread_local=False)
+        self._test_lock_token(r, lock)
+
+    def _test_lock_token(self, r, lock):
         assert lock.acquire(blocking=False, token='test')
         assert r.get('foo') == b'test'
         assert lock.local.token == b'test'
@@ -91,10 +98,13 @@ class TestLock(object):
     def test_blocking_timeout(self, r):
         lock1 = self.get_lock(r, 'foo')
         assert lock1.acquire(blocking=False)
-        lock2 = self.get_lock(r, 'foo', blocking_timeout=0.2)
+        bt = 0.2
+        sleep = 0.05
+        lock2 = self.get_lock(r, 'foo', sleep=sleep, blocking_timeout=bt)
         start = time.time()
         assert not lock2.acquire()
-        assert (time.time() - start) > 0.2
+        # The elapsed duration should be less than the total blocking_timeout
+        assert bt > (time.time() - start) > bt - sleep
         lock1.release()
 
     def test_context_manager(self, r):
@@ -110,10 +120,18 @@ class TestLock(object):
             with self.get_lock(r, 'foo', blocking_timeout=0.1):
                 pass
 
-    def test_high_sleep_raises_error(self, r):
-        "If sleep is higher than timeout, it should raise an error"
-        with pytest.raises(LockError):
-            self.get_lock(r, 'foo', timeout=1, sleep=2)
+    def test_high_sleep_small_blocking_timeout(self, r):
+        lock1 = self.get_lock(r, 'foo')
+        assert lock1.acquire(blocking=False)
+        sleep = 60
+        bt = 1
+        lock2 = self.get_lock(r, 'foo', sleep=sleep, blocking_timeout=bt)
+        start = time.time()
+        assert not lock2.acquire()
+        # the elapsed timed is less than the blocking_timeout as the lock is
+        # unattainable given the sleep/blocking_timeout configuration
+        assert bt > (time.time() - start)
+        lock1.release()
 
     def test_releasing_unlocked_lock_raises_error(self, r):
         lock = self.get_lock(r, 'foo')
@@ -138,6 +156,14 @@ class TestLock(object):
         assert 16000 < r.pttl('foo') <= 20000
         lock.release()
 
+    def test_extend_lock_replace_ttl(self, r):
+        lock = self.get_lock(r, 'foo', timeout=10)
+        assert lock.acquire(blocking=False)
+        assert 8000 < r.pttl('foo') <= 10000
+        assert lock.extend(10, replace_ttl=True)
+        assert 8000 < r.pttl('foo') <= 10000
+        lock.release()
+
     def test_extend_lock_float(self, r):
         lock = self.get_lock(r, 'foo', timeout=10.0)
         assert lock.acquire(blocking=False)
